@@ -2,12 +2,15 @@ import ipaddress
 import logging
 import os
 import ssl
+import urllib.parse
 from pathlib import Path
 from typing import Any
+from typing import Literal
 from typing import TypedDict
 
 from aioquic.h3.connection import H3_ALPN
 from aioquic.tls import CipherSuite
+from cryptography import x509
 from OpenSSL import crypto
 from OpenSSL import SSL
 
@@ -15,6 +18,7 @@ from mitmproxy import certs
 from mitmproxy import connection
 from mitmproxy import ctx
 from mitmproxy import exceptions
+from mitmproxy import http
 from mitmproxy import tls
 from mitmproxy.net import tls as net_tls
 from mitmproxy.options import CONF_BASENAME
@@ -23,10 +27,12 @@ from mitmproxy.proxy.layers import modes
 from mitmproxy.proxy.layers import quic
 from mitmproxy.proxy.layers import tls as proxy_tls
 
+logger = logging.getLogger(__name__)
+
 # We manually need to specify this, otherwise OpenSSL may select a non-HTTP2 cipher by default.
 # https://ssl-config.mozilla.org/#config=old
 
-DEFAULT_CIPHERS = (
+_DEFAULT_CIPHERS = (
     "ECDHE-ECDSA-AES128-GCM-SHA256",
     "ECDHE-RSA-AES128-GCM-SHA256",
     "ECDHE-ECDSA-AES256-GCM-SHA384",
@@ -55,6 +61,22 @@ DEFAULT_CIPHERS = (
     "DES-CBC3-SHA",
 )
 
+_DEFAULT_CIPHERS_WITH_SECLEVEL_0 = ("@SECLEVEL=0", *_DEFAULT_CIPHERS)
+
+
+def _default_ciphers(
+    min_tls_version: net_tls.Version,
+) -> tuple[str, ...]:
+    """
+    @SECLEVEL=0 is necessary for TLS 1.1 and below to work,
+    see https://github.com/pyca/cryptography/issues/9523
+    """
+    if min_tls_version in net_tls.INSECURE_TLS_MIN_VERSIONS:
+        return _DEFAULT_CIPHERS_WITH_SECLEVEL_0
+    else:
+        return _DEFAULT_CIPHERS
+
+
 # 2022/05: X509_CHECK_FLAG_NEVER_CHECK_SUBJECT is not available in LibreSSL, ignore gracefully as it's not critical.
 DEFAULT_HOSTFLAGS = (
     SSL._lib.X509_CHECK_FLAG_NO_PARTIAL_WILDCARDS  # type: ignore
@@ -106,8 +128,6 @@ class TlsConfig:
     # TODO: This addon should manage the following options itself, which are current defined in mitmproxy/options.py:
     #  - upstream_cert
     #  - add_upstream_certs_to_client_chain
-    #  - ciphers_client
-    #  - ciphers_server
     #  - key_size
     #  - certs
     #  - cert_passphrase
@@ -115,12 +135,17 @@ class TlsConfig:
     #  - ssl_verify_upstream_trusted_confdir
 
     def load(self, loader):
+        insecure_tls_min_versions = (
+            ", ".join(x.name for x in net_tls.INSECURE_TLS_MIN_VERSIONS[:-1])
+            + f" and {net_tls.INSECURE_TLS_MIN_VERSIONS[-1].name}"
+        )
         loader.add_option(
             name="tls_version_client_min",
             typespec=str,
             default=net_tls.DEFAULT_MIN_VERSION.name,
             choices=[x.name for x in net_tls.Version],
-            help=f"Set the minimum TLS version for client connections.",
+            help=f"Set the minimum TLS version for client connections. "
+            f"{insecure_tls_min_versions} are insecure.",
         )
         loader.add_option(
             name="tls_version_client_max",
@@ -134,7 +159,8 @@ class TlsConfig:
             typespec=str,
             default=net_tls.DEFAULT_MIN_VERSION.name,
             choices=[x.name for x in net_tls.Version],
-            help=f"Set the minimum TLS version for server connections.",
+            help=f"Set the minimum TLS version for server connections. "
+            f"{insecure_tls_min_versions} are insecure.",
         )
         loader.add_option(
             name="tls_version_server_max",
@@ -157,6 +183,24 @@ class TlsConfig:
             help="Use a specific elliptic curve for ECDHE key exchange on server connections. "
             'OpenSSL syntax, for example "prime256v1" (see `openssl ecparam -list_curves`).',
         )
+        loader.add_option(
+            name="request_client_cert",
+            typespec=bool,
+            default=False,
+            help=f"Requests a client certificate (TLS message 'CertificateRequest') to establish a mutual TLS connection between client and mitmproxy (combined with 'client_certs' option for mitmproxy and upstream).",
+        )
+        loader.add_option(
+            "ciphers_client",
+            str | None,
+            None,
+            "Set supported ciphers for client <-> mitmproxy connections using OpenSSL syntax.",
+        )
+        loader.add_option(
+            "ciphers_server",
+            str | None,
+            None,
+            "Set supported ciphers for mitmproxy <-> server connections using OpenSSL syntax.",
+        )
 
     def tls_clienthello(self, tls_clienthello: tls.ClientHelloData):
         conn_context = tls_clienthello.context
@@ -179,7 +223,9 @@ class TlsConfig:
         if not client.cipher_list and ctx.options.ciphers_client:
             client.cipher_list = ctx.options.ciphers_client.split(":")
         # don't assign to client.cipher_list, doesn't need to be stored.
-        cipher_list = client.cipher_list or DEFAULT_CIPHERS
+        cipher_list = client.cipher_list or _default_ciphers(
+            net_tls.Version[ctx.options.tls_version_client_min]
+        )
 
         if ctx.options.add_upstream_certs_to_client_chain:  # pragma: no cover
             # exempted from coverage until https://bugs.python.org/issue18233 is fixed.
@@ -196,7 +242,7 @@ class TlsConfig:
             cipher_list=tuple(cipher_list),
             ecdh_curve=ctx.options.tls_ecdh_curve_client,
             chain_file=entry.chain_file,
-            request_client_cert=False,
+            request_client_cert=ctx.options.request_client_cert,
             alpn_select_callback=alpn_select_callback,
             extra_chain_certs=tuple(extra_chain_certs),
             dhparams=self.certstore.dhparams,
@@ -269,7 +315,9 @@ class TlsConfig:
         if not server.cipher_list and ctx.options.ciphers_server:
             server.cipher_list = ctx.options.ciphers_server.split(":")
         # don't assign to client.cipher_list, doesn't need to be stored.
-        cipher_list = server.cipher_list or DEFAULT_CIPHERS
+        cipher_list = server.cipher_list or _default_ciphers(
+            net_tls.Version[ctx.options.tls_version_server_min]
+        )
 
         client_cert: str | None = None
         if ctx.options.client_certs:
@@ -314,7 +362,9 @@ class TlsConfig:
             except ValueError:
                 host_name = server.sni.encode("idna")
                 tls_start.ssl_conn.set_tlsext_host_name(host_name)
-                ok = SSL._lib.X509_VERIFY_PARAM_set1_host(param, host_name, len(host_name))  # type: ignore
+                ok = SSL._lib.X509_VERIFY_PARAM_set1_host(  # type: ignore
+                    param, host_name, len(host_name)
+                )  # type: ignore
                 SSL._openssl_assert(ok == 1)  # type: ignore
             else:
                 # RFC 6066: Literal IPv4 and IPv6 addresses are not permitted in "HostName",
@@ -438,7 +488,7 @@ class TlsConfig:
                 else None,
             )
             if self.certstore.default_ca.has_expired():
-                logging.warning(
+                logger.warning(
                     "The mitmproxy certificate authority has expired!\n"
                     "Please delete all CA-related files in your ~/.mitmproxy folder.\n"
                     "The CA will be regenerated automatically after restarting mitmproxy.\n"
@@ -481,36 +531,130 @@ class TlsConfig:
                             f"Invalid ECDH curve: {ecdh_curve!r}"
                         ) from e
 
+        if "tls_version_client_min" in updated:
+            self._warn_unsupported_version("tls_version_client_min", True)
+        if "tls_version_client_max" in updated:
+            self._warn_unsupported_version("tls_version_client_max", False)
+        if "tls_version_server_min" in updated:
+            self._warn_unsupported_version("tls_version_server_min", True)
+        if "tls_version_server_max" in updated:
+            self._warn_unsupported_version("tls_version_server_max", False)
+        if "tls_version_client_min" in updated or "ciphers_client" in updated:
+            self._warn_seclevel_missing("client")
+        if "tls_version_server_min" in updated or "ciphers_server" in updated:
+            self._warn_seclevel_missing("server")
+
+    def _warn_unsupported_version(self, attribute: str, warn_unbound: bool):
+        val = net_tls.Version[getattr(ctx.options, attribute)]
+        supported_versions = [
+            v for v in net_tls.Version if net_tls.is_supported_version(v)
+        ]
+        supported_versions_str = ", ".join(v.name for v in supported_versions)
+
+        if val is net_tls.Version.UNBOUNDED:
+            if warn_unbound:
+                logger.info(
+                    f"{attribute} has been set to {val.name}. Note that your "
+                    f"OpenSSL build only supports the following TLS versions: {supported_versions_str}"
+                )
+        elif val not in supported_versions:
+            logger.warning(
+                f"{attribute} has been set to {val.name}, which is not supported by the current OpenSSL build. "
+                f"The current build only supports the following versions: {supported_versions_str}"
+            )
+
+    def _warn_seclevel_missing(self, side: Literal["client", "server"]) -> None:
+        """
+        OpenSSL cipher spec need to specify @SECLEVEL for old TLS versions to work,
+        see https://github.com/pyca/cryptography/issues/9523.
+        """
+        if side == "client":
+            custom_ciphers = ctx.options.ciphers_client
+            min_tls_version = ctx.options.tls_version_client_min
+        else:
+            custom_ciphers = ctx.options.ciphers_server
+            min_tls_version = ctx.options.tls_version_server_min
+
+        if (
+            custom_ciphers
+            and net_tls.Version[min_tls_version] in net_tls.INSECURE_TLS_MIN_VERSIONS
+            and "@SECLEVEL=0" not in custom_ciphers
+        ):
+            logger.warning(
+                f'With tls_version_{side}_min set to {min_tls_version}, ciphers_{side} must include "@SECLEVEL=0" '
+                f"for insecure TLS versions to work."
+            )
+
+    def crl_path(self) -> str:
+        return f"/mitmproxy-{self.certstore.default_ca.serial}.crl"
+
     def get_cert(self, conn_context: context.Context) -> certs.CertStoreEntry:
         """
         This function determines the Common Name (CN), Subject Alternative Names (SANs) and Organization Name
         our certificate should have and then fetches a matching cert from the certstore.
         """
-        altnames: list[str] = []
+        altnames: list[x509.GeneralName] = []
         organization: str | None = None
+        crl_distribution_point: str | None = None
 
         # Use upstream certificate if available.
         if ctx.options.upstream_cert and conn_context.server.certificate_list:
-            upstream_cert = conn_context.server.certificate_list[0]
+            upstream_cert: certs.Cert = conn_context.server.certificate_list[0]
             if upstream_cert.cn:
-                altnames.append(upstream_cert.cn)
+                altnames.append(_ip_or_dns_name(upstream_cert.cn))
             altnames.extend(upstream_cert.altnames)
             if upstream_cert.organization:
                 organization = upstream_cert.organization
 
+            # Replace original URL path with the CA cert serial number, which acts as a magic token
+            if crls := upstream_cert.crl_distribution_points:
+                try:
+                    scheme, netloc, *_ = urllib.parse.urlsplit(crls[0])
+                except ValueError:
+                    logger.info(f"Failed to parse CRL URL: {crls[0]!r}")
+                else:
+                    # noinspection PyTypeChecker
+                    crl_distribution_point = urllib.parse.urlunsplit(
+                        (scheme, netloc, self.crl_path(), None, None)
+                    )
+
         # Add SNI or our local IP address.
         if conn_context.client.sni:
-            altnames.append(conn_context.client.sni)
+            altnames.append(_ip_or_dns_name(conn_context.client.sni))
         else:
-            altnames.append(conn_context.client.sockname[0])
+            altnames.append(_ip_or_dns_name(conn_context.client.sockname[0]))
 
         # If we already know of a server address, include that in the SANs as well.
         if conn_context.server.address:
-            altnames.append(conn_context.server.address[0])
+            altnames.append(_ip_or_dns_name(conn_context.server.address[0]))
 
         # only keep first occurrence of each hostname
         altnames = list(dict.fromkeys(altnames))
 
         # RFC 2818: If a subjectAltName extension of type dNSName is present, that MUST be used as the identity.
         # In other words, the Common Name is irrelevant then.
-        return self.certstore.get_cert(altnames[0], altnames, organization)
+        cn = next((str(x.value) for x in altnames), None)
+        return self.certstore.get_cert(
+            cn, altnames, organization, crl_distribution_point
+        )
+
+    def request(self, flow: http.HTTPFlow):
+        if not flow.live or flow.error or flow.response:
+            return
+        # Check if a request has a magic CRL token at the end
+        if flow.request.path.endswith(self.crl_path()):
+            flow.response = http.Response.make(
+                200,
+                self.certstore.default_crl,
+                {"Content-Type": "application/pkix-crl"},
+            )
+
+
+def _ip_or_dns_name(val: str) -> x509.GeneralName:
+    """Convert a string into either an x509.IPAddress or x509.DNSName object."""
+    try:
+        ip = ipaddress.ip_address(val)
+    except ValueError:
+        return x509.DNSName(val.encode("idna").decode())
+    else:
+        return x509.IPAddress(ip)
